------------------------------------------------------------------------------
--  Ada95 Interface to Oracle RDBMS                                         --
--  Copyright (C) 2000-2003 Dmitriy Anisimkov.                              --
--  License agreement and authors contact information are in file oci.ads   --
------------------------------------------------------------------------------

--  $Id: oci-thick.ads,v 1.10 2003/11/26 10:56:12 vagul Exp $

with
   Ada.Finalization,
   Ada.Task_Identification,
   System,
   OCI.Lib,
   OCI.Reference_Finalization;

package OCI.Thick is

   package RF renames Reference_Finalization;

   use Lib;
   use Ada.Task_Identification;

   subtype OCINumber is Lib.OCINumber;

   type Limited_Variable is tagged limited private;

   procedure Clear_Value (Var : in out Limited_Variable);

   function Is_Attached (Var : in Limited_Variable) return Boolean;
   pragma Inline (Is_Attached);

   function Is_Binded (Var : in Limited_Variable) return Boolean;
   pragma Inline (Is_Binded);

   function Is_Defined (Var : in Limited_Variable) return Boolean;
   pragma Inline (Is_Defined);

   function Is_Null (Var : in Limited_Variable) return Boolean;

   Invalid_Handle : exception;
   Lib_Error      : exception;
   Null_Value     : exception;

   Not_Attached   : exception;
   --  trying get not attached variable

   Too_Late_For_Dedicated_Environment : exception;

   function Last_Error_Code return Integer;

   function Last_Error_Message (Record_No : Integer := 1) return String;

   function Last_Error_Offset return Natural;
   --  Return position of last error in the DML statement.

   type Handle_Reference is abstract new RF.Controlled_Reference with private;

   procedure Check_Error (Code : SWord; Raise_Exception : Boolean := True);

   procedure Task_Dedicated_Environment;

   procedure Task_Done (T : Task_Id := Current_Task);
   --  Call it, when you are sure about using the task is done.

   procedure Set_Events_Support;
   --  Set the Event support.
   --  Should call before first OCI call.

   procedure Set_Objects_Support;
   --  Set the object support.
   --  Should call before first OCI call.

   function Handle (Ref : Handle_Reference'Class) return System.Address;

   pragma Inline (Handle);

private

   type Handle_Reference is abstract new RF.Controlled_Reference with record
      Handle : OCIHandle := Empty_Handle;
   end record;

   package AF renames Ada.Finalization;

   type Limited_Variable is new AF.Limited_Controlled with record
      Indicator : aliased Sb2       := Null_Indicator;
      Bind      : aliased OCIBind   := OCIBind (Empty_Handle);
      Define    : aliased OCIDefine := OCIDefine (Empty_Handle);
   end record;

   procedure Free (H : in out OCIHandle; HType : Ub4);
   function Alloc_Handle (Parent : OCIEnv; Htype : Ub4) return OCIHandle;
   function Alloc_Descriptor (Parent : OCIEnv; Htype : Ub4) return OCIHandle;

   function Get_Attr
     (Param : in OCIHandle;
      HType : in Ub4;
      Attr  : in Ub4)
      return String;

   generic
      type Result_Type is (<>);
   function Get_Attr_G
     (Param  : in OCIHandle;
      HType  : in Ub4;
      Attr   : in Ub4)
      return Result_Type;
   pragma Inline (Get_Attr_G);

   procedure Set_Attr
     (H     : in OCIHandle;
      HType : in Ub4;
      Value : in String;
      Attr  : in Ub4);

   procedure Set_Attr
     (H     : in OCIHandle;
      HType : in Ub4;
      Value : in OCIHandle;
      Attr  : in Ub4);

   procedure Set_Attr
     (H     : in OCIHandle;
      HType : in Ub4;
      Value : in Ub1;
      Attr  : in Ub4);

end OCI.Thick;